/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.tier1;

import java.util.Optional;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2248;
import net.minecraft.class_2350;
import net.minecraft.class_3861;
import net.minecraft.class_3956;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.blocks.BlockMachineBase;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.recipes.RecipeCrafter;
import reborncore.common.util.ItemUtils;
import reborncore.common.util.RebornInventory;
import techreborn.config.TechRebornConfig;
import techreborn.init.TRContent;
import techreborn.init.TRBlockEntities;

public class ElectricFurnaceBlockEntity extends PowerAcceptorBlockEntity
		implements IToolDrop, InventoryProvider, IContainerProvider {

	public RebornInventory<ElectricFurnaceBlockEntity> inventory = new RebornInventory<>(3, "ElectricFurnaceBlockEntity", 64, this);
	int inputSlot = 0;
	int outputSlot = 1;
	int ticksSinceLastChange;
	private class_3861 currentRecipe;
	private int cookTime;
	private int cookTimeTotal;
	// Energy cost per tick of cooking
	final int EnergyPerTick = 4;
	
	public ElectricFurnaceBlockEntity() {
		super(TRBlockEntities.ELECTRIC_FURNACE );
	}
	
	private void setInvDirty(boolean isDirty) {
		inventory.setChanged(isDirty);
	}
	
	private boolean isInvDirty() {
		return inventory.hasChanged();
	}
	
	private void updateCurrentRecipe() {
		if (inventory.method_5438(inputSlot).method_7960()) {
			resetCrafter();
			return;
		}
		Optional<class_3861> testRecipe = field_11863.method_8433().method_8132(class_3956.field_17546, inventory, field_11863);
		if (!testRecipe.isPresent()) {
			resetCrafter();
			return;
		}
		if (!canAcceptOutput(testRecipe.get(), outputSlot)) {
			resetCrafter();
		}
		currentRecipe = testRecipe.get();
		cookTime = 0;
		cookTimeTotal = Math.max((int) (currentRecipe.method_8167() * (1.0 - getSpeedMultiplier())), 1);
		updateState();
	}

	private boolean canAcceptOutput(class_3861 recipe, int slot) {
		class_1799 recipeOutput = recipe.method_8110();
		if (recipeOutput.method_7960()) {
			return false;
		}
		if (inventory.method_5438(slot).method_7960()) {
			return true;
		}
		if (ItemUtils.isItemEqual(inventory.method_5438(slot), recipeOutput, true, true)) {
			if (recipeOutput.method_7947() + inventory.method_5438(slot).method_7947() <= recipeOutput.method_7914()) {
				return true;
			}
		}
		return false;
	}
	
	public boolean canCraftAgain() {
		if (inventory.method_5438(inputSlot).method_7960()) {
			return false;
		}
		if (currentRecipe == null) {
			return false;
		}
		if (!canAcceptOutput(currentRecipe, outputSlot)) {
			return false;
		}
		if (getEnergy() < currentRecipe.method_8167() * getEuPerTick(EnergyPerTick)) {
			return false;
		}
		
		return true;
	}
	
	private void resetCrafter() {
		currentRecipe = null;
		cookTime = 0;
		cookTimeTotal = 0;
		updateState();
	}
	
	private void updateState() {
		class_2248 furnaceBlock = method_10997().method_8320(field_11867).method_11614();

		if (furnaceBlock instanceof BlockMachineBase) {
			BlockMachineBase blockMachineBase = (BlockMachineBase) furnaceBlock;
			boolean isActive = currentRecipe != null || canCraftAgain();
			blockMachineBase.setActive(isActive, field_11863, field_11867);
		}
		field_11863.method_8413(field_11867, field_11863.method_8320(field_11867), field_11863.method_8320(field_11867), 3);
	}
	
	private boolean hasAllInputs(class_3861 recipe) {
		if (recipe == null) {
			return false;
		}
		if (inventory.method_5438(inputSlot).method_7960()) {
			return false;
		}
		
		if (!recipe.method_8115(inventory, field_11863)) {
			return false;
		}
	
		return true;
	}
	
	private void craftRecipe(class_3861 recipe) {
		if (recipe == null) {
			return;
		}
		if (!canAcceptOutput(recipe, outputSlot)) {
			return;
		}
		class_1799 outputStack = inventory.method_5438(outputSlot);
		if (outputStack.method_7960()) {
			inventory.method_5447(outputSlot, recipe.method_8110().method_7972());
		}
		else {
			// Just increment. We already checked stack match and stack size
			outputStack.method_7933(1);
		}
		
		inventory.method_5438(inputSlot).method_7934(1);
	}
	
	public int getProgressScaled(int scale) {
		if (cookTimeTotal != 0) {
			return cookTime * scale / cookTimeTotal;
		}
		return 0;		
	}
	
	public int getCookTime() {
		return cookTime;
	}

	public void setCookTime(int cookTime) {
		this.cookTime = cookTime;
	}
	
	public int getCookTimeTotal() {
		return cookTimeTotal;
	}

	public void setCookTimeTotal(int cookTimeTotal) {
		this.cookTimeTotal = cookTimeTotal;
	}
	// TilePowerAcceptor
	@Override
	public void method_16896() {
		super.method_16896();
		charge(2);
		
		if (field_11863.field_9236) {
			return;
		}

		ticksSinceLastChange++;
		// Force a has chanced every second
		if (ticksSinceLastChange == 20) {
			setInvDirty(true);
			ticksSinceLastChange = 0;
		}

		if (isInvDirty()) {
			if (currentRecipe == null) {
				updateCurrentRecipe();	
			}
			if (currentRecipe != null && (!hasAllInputs(currentRecipe) || !canAcceptOutput(currentRecipe, outputSlot))) {
				resetCrafter();
			}
		}

		if (currentRecipe != null) {
			if (cookTime >= cookTimeTotal && hasAllInputs(currentRecipe)) {
				craftRecipe(currentRecipe);
				updateCurrentRecipe();
			} else if (cookTime < cookTimeTotal) {
				if (canUseEnergy(getEuPerTick(EnergyPerTick))) {
					useEnergy(getEuPerTick(EnergyPerTick));
					cookTime++;
					if (cookTime == 1 || cookTime % 20 == 0 && RecipeCrafter.soundHanlder != null) {
						RecipeCrafter.soundHanlder.playSound(false, this);
					}
				}
			}
		}
		setInvDirty(false);
	}

	@Override
	public double getBaseMaxPower() {
		return TechRebornConfig.electricFurnaceMaxEnergy;
	}

	@Override
	public boolean canAcceptEnergy(final class_2350 direction) {
		return true;
	}

	@Override
	public boolean canProvideEnergy(final class_2350 direction) {
		return false;
	}

	@Override
	public double getBaseMaxOutput() {
		return 0;
	}

	@Override
	public double getBaseMaxInput() {
		return TechRebornConfig.electricFurnaceMaxInput;
	}

	// IToolDrop
	@Override
	public class_1799 getToolDrop(final class_1657 entityPlayer) {
		return TRContent.Machine.ELECTRIC_FURNACE.getStack();
	}

	// ItemHandlerProvider
	@Override
	public RebornInventory<ElectricFurnaceBlockEntity> getInventory() {
		return inventory;
	}

	// IContainerProvider
	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("electricfurnace").player(player.field_7514).inventory().hotbar().addInventory()
				.blockEntity(this).slot(0, 55, 45).outputSlot(1, 101, 45).energySlot(2, 8, 72).syncEnergyValue()
				.syncIntegerValue(this::getCookTime, this::setCookTime).syncIntegerValue(this::getCookTimeTotal, this::setCookTimeTotal).addInventory().create(this, syncID);
	}
}
