/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.transformers;

import reborncore.api.IListInfoProvider;
import reborncore.api.IToolDrop;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.powerSystem.PowerSystem;
import reborncore.common.util.StringUtils;
import team.reborn.energy.EnergySide;
import team.reborn.energy.EnergyTier;
import techreborn.blocks.transformers.BlockTransformer;
import techreborn.config.TechRebornConfig;

import java.util.List;
import net.minecraft.class_124;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2248;
import net.minecraft.class_2350;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2591;

/**
 * Created by Rushmead
 */
public class TransformerBlockEntity extends PowerAcceptorBlockEntity
		implements IToolDrop, IListInfoProvider {


	public String name;
	public class_2248 wrenchDrop;
	public EnergyTier inputTier;
	public EnergyTier ouputTier;
	public int maxInput;
	public int maxOutput;
	public int maxStorage;

	public TransformerBlockEntity(class_2591<?> blockEntityType, String name, class_2248 wrenchDrop, EnergyTier tier) {
		super(blockEntityType);
		this.wrenchDrop = wrenchDrop;
		this.inputTier = tier;
		if (tier != EnergyTier.MICRO) {
			ouputTier = EnergyTier.values()[tier.ordinal() - 1];
		} else {
			ouputTier = EnergyTier.MICRO;
		}
		this.name = name;
		this.maxInput = tier.getMaxInput();
		this.maxOutput = tier.getMaxOutput();
		this.maxStorage = tier.getMaxInput() * 2;

		// Should always be 4, except if we're tier MICRO, in which it will be 1.
		super.setMaxPacketsPerTick(tier.getMaxOutput() / ouputTier.getMaxInput());
	}
	
	// TilePowerAcceptor
	@Override
	public double getBaseMaxPower() {
		return maxStorage;
	}

	@Override
	public boolean canAcceptEnergy(class_2350 direction) {
		if (TechRebornConfig.IC2TransformersStyle){
			return getFacingEnum() == direction;
		}
		return getFacingEnum() != direction;
	}
	
	@Override
	public boolean canProvideEnergy(class_2350 direction) {
		if (TechRebornConfig.IC2TransformersStyle){
			return getFacingEnum() != direction;
		}
		return getFacing() == direction;
	}

	@Override
	public double getBaseMaxOutput() {
		return ouputTier.getMaxOutput();
	}

	@Override
	public double getBaseMaxInput() {
		return inputTier.getMaxInput();
	}
	
	@Override
	public EnergyTier getPushingTier() {
		return ouputTier;
	}

	@Override
	public void checkTier() {
		//Nope
	}
	
	// TileMachineBase
	@Override
	public class_2350 getFacingEnum() {
		if(field_11863 == null){
			return null;
		}
		class_2248 block = field_11863.method_8320(field_11867).method_11614();
		if (block instanceof BlockTransformer) {
			return ((BlockTransformer) block).getFacing(field_11863.method_8320(field_11867));
		}
		return null;
	}

	// IToolDrop
	@Override
	public class_1799 getToolDrop(class_1657 playerIn) {
		return new class_1799(wrenchDrop);
	}

	// IListInfoProvider
	@Override
	public void addInfo(List<class_2561> info, boolean isReal, boolean hasData) {
		info.add(new class_2585(class_124.field_1080 + "Input Rate: " + class_124.field_1065 + PowerSystem.getLocaliszedPowerFormatted((int) getMaxInput(EnergySide.UNKNOWN))));
		info.add(new class_2585(class_124.field_1080 + "Input Tier: " + class_124.field_1065 + StringUtils.toFirstCapitalAllLowercase(inputTier.toString())));
		info.add(new class_2585(class_124.field_1080 + "Output Rate: " + class_124.field_1065 + PowerSystem.getLocaliszedPowerFormatted((int) getMaxOutput(EnergySide.UNKNOWN))));
		info.add(new class_2585(class_124.field_1080 + "Output Tier: " + class_124.field_1065 + StringUtils.toFirstCapitalAllLowercase(ouputTier.toString())));
	}

}
