package techreborn.compat.libcd;

import io.github.cottonmc.libcd.tweaker.*;
import reborncore.common.crafting.RebornRecipe;
import reborncore.common.crafting.RebornRecipeType;
import reborncore.common.crafting.RecipeManager;
import reborncore.common.crafting.ingredient.FluidIngredient;
import reborncore.common.crafting.ingredient.RebornIngredient;
import reborncore.common.fluid.container.FluidInstance;
import techreborn.TechReborn;
import techreborn.api.generator.EFluidGenerator;
import techreborn.api.generator.FluidGeneratorRecipe;
import techreborn.api.generator.GeneratorRecipeHelper;
import techreborn.api.recipe.recipes.*;
import techreborn.init.ModRecipes;
import techreborn.init.TRContent;
import techreborn.items.ItemDynamicCell;

import java.util.*;
import java.util.concurrent.Executor;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1856;
import net.minecraft.class_1869;
import net.minecraft.class_2371;
import net.minecraft.class_2378;
import net.minecraft.class_2960;
import net.minecraft.class_3300;
import net.minecraft.class_3611;
import net.minecraft.class_3612;

public class TRTweaker implements Tweaker {
    public static final TRTweaker INSTANCE = new TRTweaker();
    private RecipeTweaker tweaker = RecipeTweaker.INSTANCE;

    List<FluidGeneratorRecipe> added = new ArrayList<>();
    List<FluidGeneratorRecipe> toAdd = new ArrayList<>();

    @Override
    public void prepareReload(class_3300 resourceManager) {
        for (FluidGeneratorRecipe recipe : added) {
            GeneratorRecipeHelper.removeFluidRecipe(recipe.getGeneratorType(), recipe.getFluid());
        }
        added.clear();
    }

    @Override
    public void applyReload(class_3300 resourceManager, Executor executor) {
        for (FluidGeneratorRecipe recipe : toAdd) {
            if (GeneratorRecipeHelper.getFluidRecipesForGenerator(recipe.getGeneratorType()).addRecipe(recipe)) {
                added.add(recipe);
            } else {
                TechReborn.LOGGER.error("Could not add recipe to TechReborn generator " + recipe.getGeneratorType().getRecipeID()
                        + ": a recipe for fluid " + class_2378.field_11154.method_10221(recipe.getFluid()) + " already exists");
            }
        }
        toAdd.clear();
    }

    @Override
    public String getApplyMessage() {
        int recipeCount = added.size();
        return recipeCount + " TechReborn fluid generator " + (recipeCount == 1? "recipe" : "recipes");
    }

    /**
     * Create a fluid ingredient
     * @param fluid The fluid required.
     * @param holders The fluid-holding items the fluid can be in, or [] for any.
     * @param amount The amount of fluid needed, or -1 for any.
     * @return A prepared fluid ingredient.
     */
    public FluidIngredient createFluidIngredient(String fluid, String[] holders, int amount) {
        class_3611 parsedFluid = TweakerUtils.INSTANCE.getFluid(fluid);
        Optional<List<class_1792>> parsedHolders;
        Optional<Integer> count;
        if (holders.length == 0) parsedHolders = Optional.empty();
        else {
            List<class_1792> items = new ArrayList<>();
            for (String holder : holders) {
                items.add(TweakerUtils.INSTANCE.getItem(holder));
            }
            parsedHolders = Optional.of(items);
        }
        if (amount == -1) count = Optional.empty();
        else count = Optional.of(amount);
        return new FluidIngredient(parsedFluid, parsedHolders, count);
    }

    /**
     * Register a generic TechReborn recipe.
     * @param type The type of RebornRecipe to add.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void add(RebornRecipeType type, Object[] inputs, class_1799[] outputs, int power, int time) {
        try {
            class_2960 id = tweaker.getRecipeId(outputs[0]);
            class_2371<RebornIngredient> ingredients = class_2371.method_10211();
            for (Object input : inputs) {
                ingredients.add(TRRecipeParser.processIngredient(input));
            }
            tweaker.addRecipe(new RebornRecipe(type, id, ingredients, class_2371.method_10212(class_1799.field_8037, outputs), power, time));
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn " + type.getName().method_12832().replace('_', ' ') + " recipe - " + e.getMessage());
        }
    }

    public void add(String type, Object[] inputs, class_1799[] outputs, int power, int time) {
        class_2960 id;
        if (type.contains(":")) id = new class_2960(type);
        else id = new class_2960("techreborn", type);
        RebornRecipeType<?> recipeType = RecipeManager.getRecipeType(id);
        add(recipeType, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to smelt in an alloy smelter.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to smelt for.
     */
    public void addAlloySmelter(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.ALLOY_SMELTER, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to assemble in an assembling machine.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addAssemblingMachine(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.ASSEMBLING_MACHINE, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to smelt in a TechReborn blast furnace.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to smelt for.
     * @param heat How hot the blast furnace needs to be.
     */
    public void addBlastFurnace(Object[] inputs, class_1799[] outputs, int power, int time, int heat) {
        try {
            class_2960 id = tweaker.getRecipeId(outputs[0]);
            class_2371<RebornIngredient> ingredients = class_2371.method_10211();
            for (Object input : inputs) {
                ingredients.add(TRRecipeParser.processIngredient(input));
            }
            tweaker.addRecipe(new BlastFurnaceRecipe(ModRecipes.BLAST_FURNACE, id, ingredients, class_2371.method_10212(class_1799.field_8037, outputs), power, time, heat));
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn blast furnace recipe - " + e.getMessage());
        }
    }

    /**
     * Register a recipe to process in a centrifuge.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addCentrifuge(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.CENTRIFUGE, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to process in a chemical reactor.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addChemicalReactor(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.CHEMICAL_REACTOR, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to compress in a compressor.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addCompressor(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.COMPRESSOR, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to distil in a distillation tower.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addDistillationTower(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.DISTILLATION_TOWER, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to extract in an extractor.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addExtractor(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.EXTRACTOR, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to process in a grinder.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addGrinder(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.GRINDER, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to process in an implosion compressor.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addImplosionCompressor(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.IMPLOSION_COMPRESSOR, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to process in an industrial electrolyzer.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addIndustrialElectrolyzer(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.INDUSTRIAL_ELECTROLYZER, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to process in an industrial grinder.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addIndustrialGrinder(Object[] inputs, class_1799[] outputs, int power, int time) {
        try {
            class_2960 id = tweaker.getRecipeId(outputs[0]);
            class_2371<RebornIngredient> ingredients = class_2371.method_10211();
            for (Object input : inputs) {
                ingredients.add(TRRecipeParser.processIngredient(input));
            }
            tweaker.addRecipe(new IndustrialGrinderRecipe(ModRecipes.INDUSTRIAL_GRINDER, id, ingredients, class_2371.method_10212(class_1799.field_8037, outputs), power, time));
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn industrial grinder recipe - " + e.getMessage());
        }
    }

    /**
     * Register a recipe to process in an industrial grinder.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     * @param fluid The fluid required for the operation, including an amount.
     */
    public void addIndustrialGrinder(Object[] inputs, class_1799[] outputs, int power, int time, String fluid) {
        try {
            class_2960 id = tweaker.getRecipeId(outputs[0]);
            class_2371<RebornIngredient> ingredients = class_2371.method_10211();
            for (Object input : inputs) {
                ingredients.add(TRRecipeParser.processIngredient(input));
            }
            FluidInstance fluidInst = TRRecipeParser.parseFluid(fluid);
            tweaker.addRecipe(new IndustrialGrinderRecipe(ModRecipes.INDUSTRIAL_GRINDER, id, ingredients, class_2371.method_10212(class_1799.field_8037, outputs), power, time, fluidInst));
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn industrial grinder recipe - " + e.getMessage());
        }
    }

    /**
     * Register a recipe to process in an industria sawmilll.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addIndustrialSawmill(Object[] inputs, class_1799[] outputs, int power, int time) {
        try {
            class_2960 id = tweaker.getRecipeId(outputs[0]);
            class_2371<RebornIngredient> ingredients = class_2371.method_10211();
            for (Object input : inputs) {
                ingredients.add(TRRecipeParser.processIngredient(input));
            }
            tweaker.addRecipe(new IndustrialSawmillRecipe(ModRecipes.INDUSTRIAL_SAWMILL, id, ingredients, class_2371.method_10212(class_1799.field_8037, outputs), power, time));
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn industrial sawmill recipe - " + e.getMessage());
        }
    }

    /**
     * Register a recipe to process in an industrial sawmill.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     * @param fluid The fluid required for this operation, including an amount.
     */
    public void addIndustrialSawmill(Object[] inputs, class_1799[] outputs, int power, int time, String fluid) {
        try {
            class_2960 id = tweaker.getRecipeId(outputs[0]);
            class_2371<RebornIngredient> ingredients = class_2371.method_10211();
            for (Object input : inputs) {
                ingredients.add(TRRecipeParser.processIngredient(input));
            }
            FluidInstance fluidInst = TRRecipeParser.parseFluid(fluid);
            tweaker.addRecipe(new IndustrialSawmillRecipe(ModRecipes.INDUSTRIAL_SAWMILL, id, ingredients, class_2371.method_10212(class_1799.field_8037, outputs), power, time, fluidInst));
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn industrial sawmill recipe - " + e.getMessage());
        }
    }

    /**
     * Register a recipe to process in a recycler.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addRecycler(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.RECYCLER, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to get from a scrapbox. Input is always a scrap box.
     * @param output The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addScrapbox(class_1799 output, int power, int time) {
        add(ModRecipes.SCRAPBOX, new String[]{"techreborn:scrap_box"}, new class_1799[]{output}, power, time);
    }

    /**
     * Register a recipe to process in a vacuum freezer.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addVacuumFreezer(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.VACUUM_FREEZER, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to process in a fluid replicator.
     * @param inputs The input ingredients for the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     * @param fluid The fluid required for this operation, including an amount.
     */
    public void addFluidReplicator(Object[] inputs, int power, int time, String fluid) {
        try {
            class_2960 id = tweaker.getRecipeId(new class_1799(TRContent.Parts.UU_MATTER));
            class_2371<RebornIngredient> ingredients = class_2371.method_10211();
            for (Object input : inputs) {
                ingredients.add(TRRecipeParser.processIngredient(input));
            }
            FluidInstance fluidInst = TRRecipeParser.parseFluid(fluid);
            tweaker.addRecipe(new FluidReplicatorRecipe(ModRecipes.FLUID_REPLICATOR, id, ingredients, class_2371.method_10211(), power, time, fluidInst));
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn fluid replicator recipe - " + e.getMessage());
        }
    }

    /**
     * Register a recipe to process in a fusion reactor.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     * @param startE The energy requried to start the reaction.
     * @param minSize The minimum size of the reactor coil ring for this reaction.
     */
    public void addFusionReactor(Object[] inputs, class_1799[] outputs, int power, int time, int startE, int minSize) {
        try {
            class_2960 id = tweaker.getRecipeId(outputs[0]);
            class_2371<RebornIngredient> ingredients = class_2371.method_10211();
            for (Object input : inputs) {
                ingredients.add(TRRecipeParser.processIngredient(input));
            }
            tweaker.addRecipe(new FusionReactorRecipe(ModRecipes.FUSION_REACTOR, id, ingredients, class_2371.method_10212(class_1799.field_8037, outputs), power, time, startE, minSize));
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn fusion reactor recipe - " + e.getMessage());
        }
    }

    /**
     * Register a rolling machine recipe from a 2D array of inputs, like a standard CraftTweaker recipe.
     * @param inputs the 2D array (array of arrays) of inputs to use.
     * @param output The output of the recipe.
     */
    public void addRollingMachine(Object[][] inputs, class_1799 output) {
        try {
            Object[] processed = RecipeParser.processGrid(inputs);
            int width = inputs[0].length;
            int height = inputs.length;
            addRollingMachine(processed, output, width, height);
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn rolling machine recipe - " + e.getMessage());
        }
    }

    /**
     * Register a rolling machine recipe from a 1D array of inputs.
     * @param inputs The input item or tag ids required in order: left to right, top to bottom.
     * @param output The output of the recipe.
     * @param width How many rows the recipe needs.
     * @param height How many columns the recipe needs.
     */
    public void addRollingMachine(Object[] inputs, class_1799 output, int width, int height) {
        class_2960 recipeId = tweaker.getRecipeId(output);
        try {
            class_2371<class_1856> ingredients = class_2371.method_10211();
            for (int i = 0; i < Math.min(inputs.length, width * height); i++) {
                Object id = inputs[i];
                if (id.equals("")) continue;
                ingredients.add(i, RecipeParser.processIngredient(id));
            }
            tweaker.addRecipe(new RollingMachineRecipe(ModRecipes.ROLLING_MACHINE, recipeId, new class_1869(recipeId, "", width, height, ingredients, output)));
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn rolling machine recipe - " + e.getMessage());
        }
    }

    /**
     * Register a rolling machine recipe from a pattern and dictionary.
     * @param pattern A crafting pattern like one you'd find in a vanilla recipe JSON.
     * @param dictionary A map of single characters to item or tag ids.
     * @param output The output of the recipe.
     */
    public void addRollingMachine(String[] pattern, Map<String, Object> dictionary, class_1799 output) {
        class_2960 recipeId = tweaker.getRecipeId(output);
        try {
            pattern = RecipeParser.processPattern(pattern);
            Map<String, class_1856> map = RecipeParser.processDictionary(dictionary);
            int x = pattern[0].length();
            int y = pattern.length;
            class_2371<class_1856> ingredients = RecipeParser.getIngredients(pattern, map, x, y);
            tweaker.addRecipe(new RollingMachineRecipe(ModRecipes.ROLLING_MACHINE, recipeId, new class_1869(recipeId, "", x, y, ingredients, output)));
        } catch (Exception e) {
            TechReborn.LOGGER.error("Error parsing TechReborn rolling machine recipe - " + e.getMessage());
        }
    }

    /**
     * Register a recipe to process in a solid canning machine.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addSolidCanningMachine(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.SOLID_CANNING_MACHINE, inputs, outputs, power, time);
    }

    /**
     * Register a recipe to process in a wire mill.
     * @param inputs The input ingredients for the recipe.
     * @param outputs The outputs of the recipe.
     * @param power How much power the recipe consumes per tick.
     * @param time How many ticks (1/20 of a second) to process for.
     */
    public void addWireMill(Object[] inputs, class_1799[] outputs, int power, int time) {
        add(ModRecipes.WIRE_MILL, inputs, outputs, power, time);
    }

    /**
     * Register a fluid to process in a fluid generator.
     * @param generator The type of generator: thermal, gas, diesel, semifluid, or plasma.
     * @param fluid The fluid to add a recipe for.
     * @param euPerMB How much EU should be generated per millibucket of fluid.
     */
    public void addFluidGenerator(String generator, String fluid, int euPerMB) {
        EFluidGenerator type;
        switch(generator.toLowerCase()) {
            case "thermal":
            case "techreborn.thermalgenerator":
                type = EFluidGenerator.THERMAL;
                break;
            case "gas":
            case "techreborn.gasgenerator":
                type = EFluidGenerator.GAS;
                break;
            case "diesel":
            case "techreborn.dieselgenerator":
                type = EFluidGenerator.DIESEL;
                break;
            case "semifluid":
            case "techreborn.semifluidgenerator":
                type = EFluidGenerator.SEMIFLUID;
                break;
            case "plasma":
            case "techreborn.plasmagenerator":
                type = EFluidGenerator.PLASMA;
                break;
            default:
                TechReborn.LOGGER.error("Error parsing TechReborn fluid generator recipe - could not find generator: " + generator);
                return;
        }
        class_3611 parsedFluid = TweakerUtils.INSTANCE.getFluid(fluid);
        if (parsedFluid == class_3612.field_15906) {
            TechReborn.LOGGER.error("Error parsing TechReborn fluid generator recipe - could not find fluid: " + fluid);
            return;
        }
        toAdd.add(new FluidGeneratorRecipe(parsedFluid, euPerMB, type));
    }

    public static void init() {
        Tweaker.addTweaker("TRTweaker", TRTweaker.INSTANCE);
        TweakerStackGetter.registerGetter(new class_2960("techreborn:cell"), (id) -> ItemDynamicCell.getCellWithFluid(class_2378.field_11154.method_10223(id)));
    }
}
