/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.events;

import net.fabricmc.fabric.api.event.client.ItemTooltipCallback;
import net.minecraft.class_124;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1836;
import net.minecraft.class_2237;
import net.minecraft.class_2248;
import net.minecraft.class_2343;
import net.minecraft.class_2378;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2586;
import net.minecraft.class_310;
import net.minecraft.class_437;
import reborncore.api.IListInfoProvider;
import reborncore.common.powerSystem.PowerSystem;
import reborncore.common.util.StringUtils;
import team.reborn.energy.Energy;
import team.reborn.energy.EnergyHolder;
import team.reborn.energy.EnergySide;
import techreborn.TechReborn;

import java.util.List;

public class StackToolTipHandler implements ItemTooltipCallback {

	public static void setup() {
		ItemTooltipCallback.EVENT.register(new StackToolTipHandler());
	}

	@Override
	public void getTooltip(class_1799 stack, class_1836 tooltipContext, List<class_2561> components) {
		class_1792 item = stack.method_7909();
		if (item instanceof IListInfoProvider) {
			((IListInfoProvider) item).addInfo(components, false, false);
		} else if (stack.method_7909() instanceof EnergyHolder) {
			class_2585 line1 = new class_2585(PowerSystem.getLocaliszedPowerFormattedNoSuffix(Energy.of(stack).getEnergy()));
			line1.method_10864("/");
			line1.method_10864(PowerSystem.getLocaliszedPowerFormattedNoSuffix(Energy.of(stack).getMaxStored()));
			line1.method_10864(" ");
			line1.method_10864(PowerSystem.getDisplayPower().abbreviation);
			line1.method_10854(class_124.field_1065);

			components.add(1, line1);

			if (class_437.hasShiftDown()) {
				int percentage = percentage(Energy.of(stack).getMaxStored(), Energy.of(stack).getEnergy());
				class_124 color = StringUtils.getPercentageColour(percentage);
				components.add(2, new class_2585(color + "" + percentage + "%" + class_124.field_1080 + " Charged"));
				// TODO: show both input and output rates
				components.add(3, new class_2585(class_124.field_1080 + "I/O Rate: " + class_124.field_1065 + PowerSystem.getLocaliszedPowerFormatted(((EnergyHolder) item).getMaxInput(EnergySide.UNKNOWN))));
			}
		} else {
			try {
				class_2248 block = class_2248.method_9503(item);
				if (block != null && (block instanceof class_2237 || block instanceof class_2343) && class_2378.field_11146.method_10221(block).method_12836().contains("techreborn")) {
					class_2586 blockEntity = ((class_2343) block).method_10123(class_310.method_1551().field_1687);
					boolean hasData = false;
					if (stack.method_7985() && stack.method_7969().method_10545("blockEntity_data")) {
						class_2487 blockEntityData = stack.method_7969().method_10562("blockEntity_data");
						blockEntity.method_11014(blockEntityData);
						hasData = true;
						components.add(new class_2585("Block data contained").method_10854(class_124.field_1077));
					}
					if (blockEntity instanceof IListInfoProvider) {
						((IListInfoProvider) blockEntity).addInfo(components, false, hasData);
					}
				}
			} catch (NullPointerException e) {
				TechReborn.LOGGER.debug("Failed to load info for " + stack.method_7964());
			}
		}
	}

	public int percentage(int MaxValue, int CurrentValue) {
		if (CurrentValue == 0)
			return 0;
		return (int) ((CurrentValue * 100.0f) / MaxValue);
	}

	public int percentage(double MaxValue, double CurrentValue) {
		if (CurrentValue == 0)
			return 0;
		return (int) ((CurrentValue * 100.0f) / MaxValue);
	}

}
